@@ -15,6 +15,7 @@
 
 import base64
 from hashlib import sha1
+from hashlib import sha512
 import hmac
 from httplib import HTTPConnection
 from httplib import HTTPSConnection
@@ -50,6 +51,8 @@ from swift.common.middleware.acl import referrer_allowed
 from swift.common.utils import cache_from_env
 from swift.common.utils import get_logger
 from swift.common.utils import get_remote_client
+from swift.common.utils import HASH_PATH_PREFIX
+from swift.common.utils import HASH_PATH_SUFFIX
 from swift.common.utils import split_path
 from swift.common.utils import TRUE_VALUES
 from swift.common.utils import urlparse
@@ -78,10 +81,7 @@ class Swauth(object):
         if not swift_version.at_least(SWIFT_MIN_VERSION):
             msg = ("Your Swift installation is too old (%s). You need at "
                    "least %s." % (swift.__version__, SWIFT_MIN_VERSION))
-            try:
-                self.logger.critical(msg)
-            except Exception:
-                pass
+            self.logger.critical(msg)
             raise ValueError(msg)
         self.log_headers = conf.get('log_headers', 'no').lower() in TRUE_VALUES
         self.reseller_prefix = conf.get('reseller_prefix', 'AUTH').strip()
@@ -99,20 +99,14 @@ class Swauth(object):
             self.swauth_remote = self.swauth_remote.rstrip('/')
             if not self.swauth_remote:
                 msg = _('Invalid swauth_remote set in conf file! Exiting.')
-                try:
-                    self.logger.critical(msg)
-                except Exception:
-                    pass
+                self.logger.critical(msg)
                 raise ValueError(msg)
             self.swauth_remote_parsed = urlparse(self.swauth_remote)
             if self.swauth_remote_parsed.scheme not in ('http', 'https'):
                 msg = _('Cannot handle protocol scheme %(schema)s '
                         'for url %(url)s!') % \
                    (self.swauth_remote_parsed.scheme, repr(self.swauth_remote))
-                try:
-                    self.logger.critical(msg)
-                except Exception:
-                    pass
+                self.logger.critical(msg)
                 raise ValueError(msg)
         self.swauth_remote_timeout = int(conf.get('swauth_remote_timeout', 10))
         self.auth_account = '%s.auth' % self.reseller_prefix
@@ -146,10 +140,7 @@ class Swauth(object):
         if not self.super_admin_key and not self.swauth_remote:
             msg = _('No super_admin_key set in conf file; Swauth '
                     'administration features will be disabled.')
-            try:
-                self.logger.warning(msg)
-            except Exception:
-                pass
+            self.logger.warning(msg)
         self.token_life = int(conf.get('token_life', 86400))
         self.max_token_life = int(conf.get('max_token_life', self.token_life))
         self.timeout = int(conf.get('node_timeout', 10))
@@ -241,7 +232,7 @@ class Swauth(object):
                                                             start_response)
             elif env.get('PATH_INFO', '').startswith(self.auth_prefix):
                 return self.handle(env, start_response)
-        s3 = env.get('HTTP_AUTHORIZATION')
+        s3 = env.get('swift3.auth_details')
         if s3 and not self.s3_support:
             msg = 'S3 support is disabled in swauth.'
             return HTTPBadRequest(body=msg)(env, start_response)
@@ -301,6 +292,15 @@ class Swauth(object):
                 env['swift.clean_acl'] = clean_acl
         return self.app(env, start_response)
 
+    def _get_concealed_token(self, token):
+        """Returns hashed token to be used as object name in Swift.
+
+        Tokens are stored in auth account but object names are visible in Swift
+        logs. Object names are hashed from token.
+        """
+        enc_key = "%s:%s:%s" % (HASH_PATH_PREFIX, token, HASH_PATH_SUFFIX)
+        return sha512(enc_key).hexdigest()
+
     def get_groups(self, env, token):
         """Get groups for the given token.
 
@@ -322,7 +322,8 @@ class Swauth(object):
                 if expires < time():
                     groups = None
 
-        if env.get('HTTP_AUTHORIZATION'):
+        s3_auth_details = env.get('swift3.auth_details')
+        if s3_auth_details:
             if not self.s3_support:
                 self.logger.warning('S3 support is disabled in swauth.')
                 return None
@@ -333,12 +334,13 @@ class Swauth(object):
                                  'with swauth_remote mode.')
                 return None
             try:
-                account = env['HTTP_AUTHORIZATION'].split(' ')[1]
-                account, user, sign = account.split(':')
+                account, user = s3_auth_details['access_key'].split(':', 1)
+                signature_from_user = s3_auth_details['signature']
+                msg = s3_auth_details['string_to_sign']
             except Exception:
                 self.logger.debug(
-                    'Swauth cannot parse Authorization header value %r' %
-                    env['HTTP_AUTHORIZATION'])
+                    'Swauth cannot parse swift3.auth_details value %r' %
+                    (s3_auth_details, ))
                 return None
             path = quote('/v1/%s/%s/%s' % (self.auth_account, account, user))
             resp = self.make_pre_authed_request(
@@ -370,7 +372,6 @@ class Swauth(object):
                     return None
 
             password = creds_dict['hash']
-            msg = base64.urlsafe_b64decode(unquote(token))
 
             # https://bugs.python.org/issue5285
             if isinstance(password, unicode):
@@ -378,9 +379,9 @@ class Swauth(object):
             if isinstance(msg, unicode):
                 msg = msg.encode('utf-8')
 
-            s = base64.encodestring(hmac.new(password,
-                                             msg, sha1).digest()).strip()
-            if s != sign:
+            valid_signature = base64.encodestring(hmac.new(
+                password, msg, sha1).digest()).strip()
+            if signature_from_user != valid_signature:
                 return None
             groups = [g['name'] for g in detail['groups']]
             if '.admin' in groups:
@@ -409,8 +410,9 @@ class Swauth(object):
                         memcache_key, (time() + expires_from_now, groups),
                         time=expires_from_now)
             else:
+                object_name = self._get_concealed_token(token)
                 path = quote('/v1/%s/.token_%s/%s' %
-                             (self.auth_account, token[-1], token))
+                             (self.auth_account, object_name[-1], object_name))
                 resp = self.make_pre_authed_request(
                     env, 'GET', path).get_response(self.app)
                 if resp.status_int // 100 != 2:
@@ -1180,8 +1182,9 @@ class Swauth(object):
                             (path, resp.status))
         candidate_token = resp.headers.get('x-object-meta-auth-token')
         if candidate_token:
+            object_name = self._get_concealed_token(candidate_token)
             path = quote('/v1/%s/.token_%s/%s' %
-                (self.auth_account, candidate_token[-1], candidate_token))
+                (self.auth_account, object_name[-1], object_name))
             resp = self.make_pre_authed_request(
                 req.environ, 'DELETE', path).get_response(self.app)
             if resp.status_int // 100 != 2 and resp.status_int != 404:
@@ -1330,8 +1333,9 @@ class Swauth(object):
         expires = None
         candidate_token = resp.headers.get('x-object-meta-auth-token')
         if candidate_token:
+            object_name = self._get_concealed_token(candidate_token)
             path = quote('/v1/%s/.token_%s/%s' %
-                (self.auth_account, candidate_token[-1], candidate_token))
+                (self.auth_account, object_name[-1], object_name))
             delete_token = False
             try:
                 if req.headers.get('x-auth-new-token', 'false').lower() in \
@@ -1374,8 +1378,9 @@ class Swauth(object):
             # Generate new token
             token = '%stk%s' % (self.reseller_prefix, uuid4().hex)
             # Save token info
+            object_name = self._get_concealed_token(token)
             path = quote('/v1/%s/.token_%s/%s' %
-                         (self.auth_account, token[-1], token))
+                         (self.auth_account, object_name[-1], object_name))
             try:
                 token_life = min(
                     int(req.headers.get('x-auth-token-lifetime',
@@ -1451,8 +1456,9 @@ class Swauth(object):
                 if expires < time():
                     groups = None
         if not groups:
+            object_name = self._get_concealed_token(token)
             path = quote('/v1/%s/.token_%s/%s' %
-                         (self.auth_account, token[-1], token))
+                         (self.auth_account, object_name[-1], object_name))
             resp = self.make_pre_authed_request(
                 req.environ, 'GET', path).get_response(self.app)
             if resp.status_int // 100 != 2:
@@ -1578,7 +1584,7 @@ class Swauth(object):
                     and x-auth-admin-key headers which are credentials of the
                     user sending the request.
         :param user: User whose password is to be changed.
-        :returns True if user is changing his own key, False if not.
+        :returns: True if user is changing his own key, False if not.
         """
         admin_detail = self.get_admin_detail(req)
         if not admin_detail:
